use std::path::PathBuf;

use gtk::prelude::*;
use relm4::prelude::*;
use relm4::{
    gtk, Component, ComponentController, ComponentParts, ComponentSender, SimpleComponent,
};
use walkdir::WalkDir;

pub struct SheetModel {
    path: PathBuf,
}

#[relm4::component(pub)]
impl SimpleComponent for SheetModel {
    type Init = PathBuf;
    type Input = ();
    type Output = ();

    view! {
        #[root]
        gtk::Box {
            set_orientation: gtk::Orientation::Vertical,
            gtk::Label {
                set_label: &format!("{}", model.path.file_name().unwrap().to_string_lossy()),
                set_halign: gtk::Align::Start,
                set_margin_all: 10,
            }
        }
    }

    fn init(
        path: Self::Init,
        root: &Self::Root,
        _sender: ComponentSender<Self>,
    ) -> ComponentParts<Self> {
        let model = SheetModel { path };
        let widgets = view_output!();
        ComponentParts { model, widgets }
    }
}

pub struct SheetListingModel;

#[relm4::component(pub)]
impl SimpleComponent for SheetListingModel {
    type Init = PathBuf;
    type Input = ();
    type Output = ();

    view! {
        #[root]
        gtk::Box {
            set_orientation: gtk::Orientation::Vertical,
        }
    }

    fn init(
        dir: Self::Init,
        root: &Self::Root,
        _sender: ComponentSender<Self>,
    ) -> ComponentParts<Self> {
        for entry in WalkDir::new(dir)
            .into_iter()
            .filter_map(|e| e.ok())
            .filter(|file| file.file_type().is_file())
            .map(|file| file.into_path())
            .filter(|path| {
                path.extension()
                    .map(|s| s.to_string_lossy().to_ascii_lowercase() == "pdf")
                    .unwrap_or(false)
            })
        {
            let sheet_model = SheetModel::builder().launch(entry);
            root.append(sheet_model.widget());
        }

        let model = SheetListingModel;
        let widgets = view_output!();
        ComponentParts { model, widgets }
    }
}
